unit FloatMinimizer;

{******************************************************************************

 Project:			TFloatMinimizer v.1.0
 File:				FloatMinimizer.pas
 Created:			11/17/1998
 Latest:			11/18/1998
 Author:			Christophe Porteneuve aka TDD
 Copyright:		(c) 1998 Christophe Porteneuve / The Magix Team!
 Email:				tdd@heartthrob.com
 Rights:			Freeware for anyone, any use (commercial use please mention
							the author in your product credits)
							BUT for	Bryant H. McGill: $10.00 fee

 ******************************************************************************}

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	DsgnIntf;

type
	EThereCanBeOnlyOne = class(Exception);
	EOwnerMustBeForm = class(Exception);
	EOwnerMustBeMainForm = class(Exception);

	TFloatMinimizer = class(TComponent)
	private
		FAuthor: string;
		FCopyTitle: Boolean;
		FDestroying: Boolean;
		FForm: TCustomForm;
		FOldAppWndStyle: Integer;
		FOldWndProc: Pointer;
		FRestore: Boolean;
		procedure InitTrickyWindowStuff;
		procedure RestoreAppStyle;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
	published
		property CopyTitle: Boolean read FCopyTitle write FCopyTitle default True;
		property RestoreStyle: Boolean read FRestore write FRestore	default False;
		property Author: string read FAuthor write FAuthor;
	end; // TFloatMinimizer

	TAuthorProperty = class(TPropertyEditor)
	public
		procedure Edit; override;
		function GetAttributes: TPropertyAttributes; override;
		function GetValue: string; override;
	end; // TAuthorProperty

	TFloatMinimizerEditor = class(TComponentEditor)
	public
		procedure Edit; override;
		function GetVerbCount: Integer; override;
		function GetVerb(Index: Integer): string; override;
		procedure ExecuteVerb(Index: Integer); override;
	end; // TFloatMinimizerEditor

procedure Register;

implementation

var
	MinimizerVar: TFloatMinimizer;

function HookWndProc(HWindow: HWnd; Msg, WParam,
	LParam: Longint): Longint; stdcall;
begin
	if (Msg = WM_SYSCOMMAND) and
		(WParam and $FFF0 = SC_MINIMIZE) then
	begin
		MinimizerVar.FForm.WindowState := wsMinimized;
		Result := 0;
	end else
	begin
		if Msg = WM_DESTROY then
		begin
			MinimizerVar.FDestroying := True;
			SetWindowLong(MinimizerVar.FForm.Handle, GWL_WNDPROC,
				Integer(MinimizerVar.FOldWndProc));
		end;
		Result := CallWindowProc(MinimizerVar.FOldWndProc,
			HWindow, Msg, WParam, LParam);
		if MinimizerVar.FDestroying then
			MinimizerVar.FOldWndProc := nil;
	end;
end; // HookWndProc

{ TFloatMinimizer }

constructor TFloatMinimizer.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);

	if (MinimizerVar <> nil) and (MinimizerVar <> Self) then
		raise EThereCanBeOnlyOne.Create('Only one TFloatMinimizer per app!');
	MinimizerVar := Self;

	FCopyTitle := True;
	FDestroying := False;
	FForm := nil;
	FRestore := False;

	if not (csDesigning in ComponentState) then
		InitTrickyWindowStuff;
end; // Create

destructor TFloatMinimizer.Destroy;
begin
	if not (csDesigning in ComponentState) then
	begin
		if not FDestroying then
			SetWindowLong(FForm.Handle, GWL_WNDPROC,	Integer(FOldWndProc));
		if RestoreStyle then
			RestoreAppStyle;
	end else
		MinimizerVar := nil;
	inherited;
end; // Destroy

type
	TApplicationHack = class(TApplication);

procedure TFloatMinimizer.InitTrickyWindowStuff;
const
	ShowFlags: array[Boolean] of Word = (
		SWP_NOSIZE + SWP_NOMOVE + SWP_NOZORDER + SWP_NOACTIVATE + SWP_HIDEWINDOW,
		SWP_NOSIZE + SWP_NOMOVE + SWP_NOZORDER + SWP_NOACTIVATE + SWP_SHOWWINDOW);
var
	AStyle: Integer;
begin
	// Removing button for the app window, and hiding it from Alt+Tab list
	SetWindowPos(Application.Handle, 0, 0, 0, 0, 0, ShowFlags[False]);
	AStyle := GetWindowLong(Application.Handle, GWL_EXSTYLE);
	FOldAppWndStyle := AStyle;
	AStyle := AStyle or WS_EX_TOOLWINDOW and not WS_EX_APPWINDOW;
	SetWindowLong(Application.Handle, GWL_EXSTYLE, AStyle);
	SetWindowPos(Application.Handle, 0, 0, 0, 0, 0, ShowFlags[True]);

	// Some preliminary checks -- the component must be dropped on the main form
	Assert(Assigned(Owner));
	if not (Owner is TCustomForm) then
		raise EOwnerMustBeForm.CreateFmt('%s is not a form', [Owner.Name]);
	if (Application.MainForm <> nil) and (Application.MainForm <> Owner) then
		raise EOwnerMustBeMainForm.CreateFmt('%s is not the main form',
			[Owner.Name]);
	FForm := TCustomForm(Owner);

	// Adding button and appwindow style for the main form
	if CopyTitle then
		FForm.Caption := Application.Title;
	AStyle := GetWindowLong(FForm.Handle, GWL_EXSTYLE);
	AStyle := AStyle or WS_EX_APPWINDOW;
	SetWindowLong(FForm.Handle, GWL_EXSTYLE, AStyle);

	// Hooking the WndProc
	FOldWndProc := Pointer(SetWindowLong(FForm.Handle, GWL_WNDPROC,
		Integer(@HookWndProc)));
end; // InitTrickyWindowStuff

procedure TFloatMinimizer.RestoreAppStyle;
var
	CurrentStyle: Integer;
begin
	CurrentStyle := GetWindowLong(Application.Handle, GWL_EXSTYLE);
	if CurrentStyle <> FOldAppWndStyle then
		SetWindowLong(Application.Handle, GWL_EXSTYLE, FOldAppWndStyle);
end; // RestoreAppStyle

procedure ShowAuthor;
const
	AUTHOR_INFO: string =
		'TFloatMinimizer v.1.0'#13#10+
		'(c) 1998 Christophe Porteneuve / The Magix Team!'#13#10+
		'Email: tdd@heartthrob.com'#13#10+
		#13#10+
		'Freeware but for Bryant H. McGill :-)';
begin
	MessageDlg(AUTHOR_INFO, mtInformation, [mbOk], 0);
end; // ShowAuthor

{ TAuthorProperty }

procedure TAuthorProperty.Edit;
begin
	ShowAuthor;
end; // Edit

function TAuthorProperty.GetAttributes: TPropertyAttributes;
begin
	Result := [paReadOnly, paDialog];
end; // GetAttributes

function TAuthorProperty.GetValue: string;
begin
	Result := '(Double-click me!)';
end; // GetValue

{ TFloatMinimizerEditor }

procedure TFloatMinimizerEditor.Edit;
begin
	ShowAuthor;
end; // Edit

procedure TFloatMinimizerEditor.ExecuteVerb(Index: Integer);
begin
	Edit;
end; // ExecuteVerb

function TFloatMinimizerEditor.GetVerb(Index: Integer): string;
begin
	Result := 'Author...';
end; // GetVerb

function TFloatMinimizerEditor.GetVerbCount: Integer;
begin
	Result := 1;
end; // GetVerbCount

procedure Register;
begin
	RegisterComponents('TDDians', [TFloatMinimizer]);
	RegisterPropertyEditor(TypeInfo(string), TFloatMinimizer, 'Author',
		TAuthorProperty);
	RegisterComponentEditor(TFloatMinimizer, TFloatMinimizerEditor);
end;

initialization
	MinimizerVar := nil;
end.
